package com.taais.biz.service;

import com.taais.biz.domain.DataAugmentation;
import com.taais.biz.domain.bo.DataAugmentationBo;
import com.taais.biz.domain.bo.DataAugmentationResultBo;
import com.taais.biz.domain.bo.DataAugmentationStartResultBo;
import com.taais.biz.domain.bo.VideoStableStartResultBo;
import com.taais.biz.domain.vo.DataAugmentationVo;
import com.taais.common.core.core.domain.CommonResult;
import com.taais.common.core.core.page.PageResult;
import com.taais.common.orm.core.service.IBaseService;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Stream;

/**
 * 数据增强Service接口
 *
 * @author tanlun
 * 2024-08-30
 */
public interface IDataAugmentationService extends IBaseService<DataAugmentation> {
    /**
     * 查询数据增强
     *
     * @param id 数据增强主键
     * @return 数据增强
     */
        DataAugmentationVo selectById(Long id);

    /**
     * 查询数据增强列表
     *
     * @param dataAugmentationBo 数据增强Bo
     * @return 数据增强集合
     */
    List<DataAugmentationVo> selectList(DataAugmentationBo dataAugmentationBo);

    /**
     * 分页查询数据增强列表
     *
     * @param dataAugmentationBo 数据增强Bo
     * @return 分页数据增强集合
     */
    PageResult<DataAugmentationVo> selectPage(DataAugmentationBo dataAugmentationBo);

    /**
     * 新增数据增强
     *
     * @param dataAugmentationBo 数据增强Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    boolean insert(DataAugmentationBo dataAugmentationBo);

    /**
     * 修改数据增强
     *
     * @param dataAugmentationBo 数据增强Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    boolean update(DataAugmentationBo dataAugmentationBo);

    /**
     * 批量删除数据增强
     *
     * @param ids 需要删除的数据增强主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    boolean deleteByIds(Long[] ids);

    CommonResult start(Long id);

    CommonResult getResult(DataAugmentationResultBo videoStableStartResultBo);

    HashMap<String, Integer> getCompareNum(Long taskId);

    CommonResult stop(Long id);

    PageResult<DataAugmentationVo> selectPage1(DataAugmentationBo dataAugmentationBo, String type);
}
