/**
 * 20241029 ©WANGKANG 1686617586@qq.com
 */

package com.taais.biz.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.mybatisflex.core.paginate.Page;
import com.mybatisflex.core.query.QueryWrapper;
import com.taais.biz.constant.BizConstant;
import com.taais.biz.domain.*;
import com.taais.biz.domain.bo.TargetDetectionBo;
import com.taais.biz.domain.vo.AlgorithmConfigTrackVo;
import com.taais.biz.domain.vo.AlgorithmModelTrackVo;
import com.taais.biz.domain.vo.StartTaskConfig;
import com.taais.biz.domain.vo.TargetDetectionVo;
import com.taais.biz.mapper.TargetDetectionMapper;
import com.taais.biz.service.ITargetDetectionService;
import com.taais.biz.utils.ZipUtils;
import com.taais.common.core.config.TaaisConfig;
import com.taais.common.core.constant.Constants;
import com.taais.common.core.core.domain.CommonResult;
import com.taais.common.core.core.page.PageResult;
import com.taais.common.core.utils.MapstructUtils;
import com.taais.common.core.utils.StringUtils;
import com.taais.common.orm.core.page.PageQuery;
import com.taais.common.orm.core.service.impl.BaseServiceImpl;
import com.taais.common.websocket.utils.WebSocketUtils;
import com.taais.system.domain.vo.SysOssVo;
import com.taais.system.service.ISysOssService;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.util.*;

import static com.taais.biz.constant.BizConstant.VideoStatus.NOT_START;
import static com.taais.biz.domain.table.TargetDetectionTableDef.TARGET_DETECTION;
import static com.taais.biz.service.impl.ToInfraredServiceImpl.readLogContent;
import static com.taais.biz.service.impl.VideoStableServiceImpl.*;

/**
 * 目标检测Service业务层处理
 *
 * @author wangkang
 * 2024-09-29
 */
@Service
public class TargetDetectionServiceImpl extends BaseServiceImpl<TargetDetectionMapper, TargetDetection> implements ITargetDetectionService {
    @Value("${server.port}")
    public static String port;

    @Value("${server.task_stop_url}")
    private String task_stop_url;

    @Resource
    private TargetDetectionMapper targetDetectionMapper;
    @Autowired
    private ISysOssService ossService;

    @Autowired
    private AlgorithmConfigTrackServiceImpl algorithmConfigTrackService;
    @Autowired
    private AlgorithmModelTrackServiceImpl algorithmModelTrackService;

    @Override
    public QueryWrapper query() {
        return super.query().from(TARGET_DETECTION);
    }

    private QueryWrapper buildQueryWrapper(TargetDetectionBo targetDetectionBo) {
        QueryWrapper queryWrapper = super.buildBaseQueryWrapper();
        queryWrapper.and(TARGET_DETECTION.NAME.like(targetDetectionBo.getName()));
        queryWrapper.and(TARGET_DETECTION.STATUS.eq(targetDetectionBo.getStatus()));
        queryWrapper.and(TARGET_DETECTION.START_TIME.eq(targetDetectionBo.getStartTime()));
        queryWrapper.and(TARGET_DETECTION.END_TIME.eq(targetDetectionBo.getEndTime()));
        queryWrapper.and(TARGET_DETECTION.COST_SECOND.eq(targetDetectionBo.getCostSecond()));
        queryWrapper.and(TARGET_DETECTION.LOG.eq(targetDetectionBo.getLog()));
        queryWrapper.and(TARGET_DETECTION.REMARKS.eq(targetDetectionBo.getRemarks()));
        queryWrapper.and(TARGET_DETECTION.URL.eq(targetDetectionBo.getUrl()));
        queryWrapper.and(TARGET_DETECTION.INPUT_OSS_ID.eq(targetDetectionBo.getInputOssId()));
        queryWrapper.and(TARGET_DETECTION.INPUT_PATH.eq(targetDetectionBo.getInputPath()));
        queryWrapper.and(TARGET_DETECTION.OUTPUT_PATH.eq(targetDetectionBo.getOutputPath()));
        queryWrapper.and(TARGET_DETECTION.ZIP_FILE_PATH.eq(targetDetectionBo.getZipFilePath()));
        queryWrapper.and(TARGET_DETECTION.ALGORITHM_MODEL_ID.eq(targetDetectionBo.getAlgorithmModelId()));

        return queryWrapper;
    }

    /**
     * 查询目标检测
     *
     * @param id 目标检测主键
     * @return 目标检测
     */
    @Override
    public TargetDetectionVo selectById(Long id) {
        return this.getOneAs(query().where(TARGET_DETECTION.ID.eq(id)), TargetDetectionVo.class);

    }

    /**
     * 查询目标检测列表
     *
     * @param targetDetectionBo 目标检测Bo
     * @return 目标检测集合
     */
    @Override
    public List<TargetDetectionVo> selectList(TargetDetectionBo targetDetectionBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(targetDetectionBo);
        queryWrapper.orderBy(TARGET_DETECTION.CREATE_TIME,Boolean.FALSE);
        return this.listAs(queryWrapper, TargetDetectionVo.class);
    }

    /**
     * 分页查询目标检测列表
     *
     * @param targetDetectionBo 目标检测Bo
     * @return 分页目标检测集合
     */
    @Override
    public PageResult<TargetDetectionVo> selectPage(TargetDetectionBo targetDetectionBo) {
        QueryWrapper queryWrapper = buildQueryWrapper(targetDetectionBo);
        queryWrapper.orderBy(TARGET_DETECTION.CREATE_TIME,Boolean.FALSE);
        Page<TargetDetectionVo> page = this.pageAs(PageQuery.build(), queryWrapper, TargetDetectionVo.class);
        page.getRecords().forEach(entity -> {
            Long modelId = entity.getAlgorithmModelId();
            AlgorithmModelTrackVo model = algorithmModelTrackService.selectById(modelId);
            if (ObjectUtil.isNotNull(model)) {
                AlgorithmConfigTrackVo config = algorithmConfigTrackService.selectById(model.getAlgorithmId());
                entity.setType(config.getType());
                entity.setSubsystem(config.getSubsystem());
                entity.setAlgorithmName(config.getAlgorithmName());
                entity.setModelName(model.getModelName());
            }
        });
        page.getRecords().sort(Comparator.comparing(TargetDetectionVo::getCreateTime).reversed());
        return PageResult.build(page);
    }

    /**
     * 新增目标检测
     *
     * @param targetDetectionBo 目标检测Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    @Override
    public boolean insert(TargetDetectionBo targetDetectionBo) {
        // 检查input_oss_id是否存在
        if (ObjectUtil.isNull(targetDetectionBo.getInputOssId())) {
            return false;
        }

        SysOssVo ossEntity = ossService.getById(targetDetectionBo.getInputOssId());
        if (ObjectUtil.isNull(ossEntity)) {
            return false;
        }

        TargetDetection targetDetection = new TargetDetection();

        targetDetection.setInputOssId(targetDetectionBo.getInputOssId());
        targetDetection.setUrl(ossEntity.getUrl());

        String filePath = ossEntity.getFileName();
        String localPath = TaaisConfig.getProfile();
        String resourcePath = localPath + StringUtils.substringAfter(filePath, Constants.RESOURCE_PREFIX);
        targetDetection.setInputPath(resourcePath);

        String fileName = StringUtils.substringAfterLast(filePath, "/");
        String fileName_without_suffix = removeFileExtension(fileName);

        Path path = Paths.get(resourcePath);
        Path outPath = path.resolveSibling(fileName_without_suffix + "_images" + System.currentTimeMillis());
        targetDetection.setOutputPath(outPath.toString());

        targetDetection.setZipFilePath(path.resolveSibling(fileName_without_suffix + ".zip").toString());

        targetDetection.setName(targetDetectionBo.getName());
        targetDetection.setStatus(NOT_START);
        targetDetection.setRemarks(targetDetectionBo.getRemarks());

        targetDetection.setAlgorithmModelId(targetDetectionBo.getAlgorithmModelId());

        return this.save(targetDetection);// 使用全局配置的雪花算法主键生成器生成ID值
    }

    /**
     * 修改目标检测
     *
     * @param targetDetectionBo 目标检测Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    @Override
    public boolean update(TargetDetectionBo targetDetectionBo) {
        TargetDetection targetDetection = MapstructUtils.convert(targetDetectionBo, TargetDetection.class);
        if (ObjectUtil.isNotNull(targetDetection) && ObjectUtil.isNotNull(targetDetection.getId())) {
            boolean updated = this.updateById(targetDetection);
            return updated;
        }
        return false;
    }

    /**
     * 批量删除目标检测
     *
     * @param ids 需要删除的目标检测主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    @Transactional
    @Override
    public boolean deleteByIds(Long[] ids) {
        return this.removeByIds(Arrays.asList(ids));
    }

    @Override
    public CommonResult start(Long id) {
        TargetDetection entity = getById(id);

        SysOssVo inputOssEntity = ossService.getById(entity.getInputOssId());

        String filePath = inputOssEntity.getFileName();
        String localPath = TaaisConfig.getProfile();
        String resourcePath = localPath + StringUtils.substringAfter(filePath, Constants.RESOURCE_PREFIX);

        String fileName = StringUtils.substringAfterLast(filePath, "/");
        String fileName_without_suffix = removeFileExtension(fileName);

        Path path = Paths.get(resourcePath);
        Path inputPath = path.resolveSibling(fileName_without_suffix + BizConstant.UNZIP_SUFFIX);
        Path outputPath = path.resolveSibling(entity.getId().toString() + BizConstant.TARGET_DETECTION_SUFFIX);

//        makeDir(inputPath.toString());
        makeDir(outputPath.toString());

        File file = new File(inputPath.toString());
        if (!file.exists()) {
            ZipUtils.unzip(resourcePath, inputPath.toString());
        }

        entity.setInputPath(inputPath.toString());
        entity.setOutputPath(outputPath.toString());

        entity.setStartTime(new Date());

        AlgorithmModelTrack algorithmModelTrack = algorithmModelTrackService.getById(entity.getAlgorithmModelId());
        AlgorithmConfigTrack algorithmConfigTrack = algorithmConfigTrackService.getById(algorithmModelTrack.getAlgorithmId());

        StartTaskConfig startTaskConfig = new StartTaskConfig();
        startTaskConfig.setBizType(BizConstant.BizType.TARGET_DETECTION);
        startTaskConfig.setBizId(entity.getId());

        startTaskConfig.setOtherParams(algorithmConfigTrack.getParameterConfig());

        startTaskConfig.setSource_dir(entity.getInputPath());
        startTaskConfig.setResult_dir(entity.getOutputPath());
        startTaskConfig.setLog_path(entity.getOutputPath() + File.separator + getLogFileName(entity));

        if (BizConstant.AlgorithmType.REASONING.equals(algorithmConfigTrack.getType())) {
            String modelPath = algorithmModelTrack.getModelAddress();
            startTaskConfig.setModel_path(modelPath);
        } else if (BizConstant.AlgorithmType.TRAIN.equals(algorithmConfigTrack.getType())) {
            // 这时候需要在之前的基础上，添加类别的路径
            File dir = new File(startTaskConfig.getSource_dir());
            File[] files = dir.listFiles();
            // 暂时只传第一个文件夹用作训练
            if (files == null || files.length == 0) {
                return CommonResult.fail("输入数据集为空！");
            }
            for (File file__ : files) {
                if (file__.isDirectory()) {
                    startTaskConfig.setSource_dir(file__.getAbsolutePath()); // 设置路径
                    break;
                }
            }
        }

        HttpResponseEntity responseEntity = sendPostMsg(algorithmConfigTrack.getAlgorithmAddress(), startTaskConfig);
        if (ObjectUtil.isNotNull(responseEntity) && responseEntity.getStatus() == 200) {
            entity.setStatus(BizConstant.VideoStatus.RUNNING);
            updateById(entity);
            return CommonResult.success("任务开始成功，请等待完成");
        } else {
            entity.setStatus(BizConstant.VideoStatus.FAILED);
            updateById(entity);
            return CommonResult.fail("任务开始失败，请检查！");
        }
    }

    @Override
    public CommonResult stop(Long id) {
        TargetDetection entity = getById(id);

        StartTaskConfig startTaskConfig = new StartTaskConfig();
        startTaskConfig.setBizType(BizConstant.BizType.TO_INFRARED);
        startTaskConfig.setBizId(entity.getId());

        HttpResponseEntity responseEntity = sendPostMsg(task_stop_url, startTaskConfig);
        if (ObjectUtil.isNotNull(responseEntity) && responseEntity.getStatus() == 200) {
            entity.setStatus(BizConstant.VideoStatus.INTERRUPTED);
            updateById(entity);
            return CommonResult.success("终止任务成功");
        } else {
            return CommonResult.fail("终止任务失败");
        }
    }

    @Override
    public CommonResult getResult(TaskTrackResultBo taskTrackResultBo) {
        Long id = taskTrackResultBo.getBizId();
        String status = taskTrackResultBo.getStatus();
        String msg = taskTrackResultBo.getMsg();
        TargetDetection entity = getById(id);
        entity.setLog(msg);
        entity.setStatus("200".equals(status) ? BizConstant.VideoStatus.END : BizConstant.VideoStatus.FAILED);
        entity.setEndTime(new Date());
        try {
            entity.setCostSecond((entity.getEndTime().getTime() - entity.getStartTime().getTime()) / 1000);
        } catch (Exception e) {
            entity.setCostSecond(null);
        }
        updateById(entity);

        AlgorithmModelTrack algorithmModelTrack = algorithmModelTrackService.getById(entity.getAlgorithmModelId());
        algorithmModelTrack.setModelStatus("200".equals(status) ? BizConstant.ModelStatus.END : BizConstant.ModelStatus.FAILED);

        AlgorithmConfigTrack algorithmConfigTrack = algorithmConfigTrackService.getById(algorithmModelTrack.getAlgorithmId());
        String params = algorithmConfigTrack.getParameterConfig();
        HashMap<String, Object> parse = (HashMap<String, Object>) JSON.parse((params));

        if ("200".equals(status) && ObjectUtil.isNull(algorithmModelTrack.getModelAddress())) {
            algorithmModelTrack.setModelAddress(entity.getOutputPath() + File.separator + ((HashMap<String, String>) parse.get("dataset")).get("classes"));

            System.out.println(parse.get("dataset"));
            algorithmModelTrackService.updateById(algorithmModelTrack);
        }

        /*
         * 这里有很多需要做的：1. 保存所有模型到oss 2. 模型文件保存到算法模型配置表中
         */
        /*
        Path resultsPath = Paths.get(entity.getInputPath());
        Path modelPath = resultsPath.resolve(BizConstant.MODEL_PATH);

        File modelDir = new File(modelPath.toString());

        if(!modelDir.exists() || !modelDir.isDirectory()) {
            return CommonResult.fail("模型文件不存在");
        }

        for (File f : modelDir.listFiles()) {
            if (f.isFile() && f.getName().endsWith(BizConstant.MODEL_SUFFIX)) {
                SysOssVo upload = ossService.upload(f);
                AlgorithmModelTrack algorithmModelTrack = new AlgorithmModelTrack();
                algorithmModelTrack.set
                algorithmModelTrackService.insert();
            }
        }
        */
        WebSocketUtils.publishAll("refresh");
        return CommonResult.success();
    }

    @Override
    public ResponseEntity<org.springframework.core.io.Resource> zipImages(Long id) {
        TargetDetection targetDetection = this.getById(id);
        if (ObjectUtil.isNull(targetDetection)) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(null);
        }

        String outputPath = targetDetection.getOutputPath();
        String zipFilePath = outputPath + ".zip";

        File file = new File(zipFilePath);
        if (!file.exists()) {
            try {
                ZipUtils.zipFolderFiles(outputPath, zipFilePath);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        if (!file.exists() || !file.isFile()) {
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(null);
        }

        org.springframework.core.io.Resource resource = new FileSystemResource(file);
        return ResponseEntity.ok().header(HttpHeaders.CONTENT_DISPOSITION, "attachment; filename=\"" + file.getName() + "\"").header(HttpHeaders.CONTENT_TYPE, "application/octet-stream").body(resource);
    }

    private String getLogFileName(TargetDetection entity) {
        return entity.getId() + BizConstant.TARGET_DETECTION_SUFFIX + ".log";
    }

    @Override
    public CommonResult getLog(Long id) {
        TargetDetection entity = getById(id);
        String outPutPath = entity.getOutputPath();
        String logPath = outPutPath + File.separator + getLogFileName(entity);
        System.out.println(logPath);
        File file = new File(logPath);
        if (!file.exists()) {
            return CommonResult.fail("日志文件不存在！");
        }
        return CommonResult.success(readLogContent(logPath), "success");
    }

    @Override
    public CommonResult getModelList(Long id) {
        TargetDetection entity = getById(id);
        String outPutPath = entity.getOutputPath();
        String modelPath = outPutPath;
        File modelDir = new File(modelPath);
        if (!modelDir.exists()) {
            return CommonResult.fail("模型输出目录不存在！");
        }
        File[] files = modelDir.listFiles();
        Integer idx = 0;
        ArrayList<Map<String, String>> res = new ArrayList<>();
        for (File file : files) {
            if (!file.getName().endsWith(BizConstant.TD_TRAIN_MODEL_SUFFIX)) {
                continue;
            }
            idx += 1;
            Map<String, String> tmp = new HashMap<>();
            tmp.put("id", idx.toString());
            tmp.put("name", file.getName());
            String filePath = file.getPath();
            tmp.put("path", filePath);
            // todo 获取真正的url
            // http://localhost:9090/profile/upload/2024/10/27/1_1729404909511_20241027153840A001.zip
            String url = "http://localhost:" + port + Constants.RESOURCE_PREFIX + filePath.substring(TaaisConfig.getProfile().length());
            url = url.replaceAll("\\\\", "/"); // windows
            tmp.put("url", url);

            double fileSize = (getFileSize(file) / (1024.0 * 1024.0));
            DecimalFormat decimalFormat = new DecimalFormat("#.##");
            String formatFileSize = decimalFormat.format(fileSize);
            tmp.put("size", formatFileSize + "MB");
            res.add(tmp);
        }
        return CommonResult.success(res, "success");
    }

    public static long getFileSize(File file) {
        // 返回单位B
        if (file.isFile()) {
            return file.length();
        } else {
            long totalSize = 0;
            if (file.listFiles() != null && file.listFiles().length > 0) {
                for (File file__ : Objects.requireNonNull(file.listFiles())) {
                    totalSize += getFileSize(file__);
                }
            }
            return totalSize;
        }
    }
}
