package com.taais.biz.service.service;

import java.util.List;

import com.taais.biz.domain.ObjectMatch;
import com.taais.biz.domain.bo.ObjectMatchBo;
import com.taais.biz.domain.vo.ObjectMatchVo;
import com.taais.common.orm.core.service.IBaseService;
import com.taais.common.core.core.page.PageResult;

/**
 * 异源图像匹配Service接口
 *
 * @author km
 * 2024-10-28
 */
public interface IObjectMatchService extends IBaseService<ObjectMatch> {
    /**
     * 查询异源图像匹配
     *
     * @param id 异源图像匹配主键
     * @return 异源图像匹配
     */
        ObjectMatchVo selectById(Long id);

    /**
     * 查询异源图像匹配列表
     *
     * @param objectMatchBo 异源图像匹配Bo
     * @return 异源图像匹配集合
     */
    List<ObjectMatchVo> selectList(ObjectMatchBo objectMatchBo);

    /**
     * 分页查询异源图像匹配列表
     *
     * @param objectMatchBo 异源图像匹配Bo
     * @return 分页异源图像匹配集合
     */
    PageResult<ObjectMatchVo> selectPage(ObjectMatchBo objectMatchBo);

    /**
     * 新增异源图像匹配
     *
     * @param objectMatchBo 异源图像匹配Bo
     * @return 结果:true 操作成功，false 操作失败
     */
    boolean insert(ObjectMatchBo objectMatchBo);

    /**
     * 修改异源图像匹配
     *
     * @param objectMatchBo 异源图像匹配Bo
     * @return 结果:true 更新成功，false 更新失败
     */
    boolean update(ObjectMatchBo objectMatchBo);

    /**
     * 批量删除异源图像匹配
     *
     * @param ids 需要删除的异源图像匹配主键集合
     * @return 结果:true 删除成功，false 删除失败
     */
    boolean deleteByIds(Long[] ids);

}
